#import "EBGFView.h"

@implementation EBGFView

@synthesize delegate;

/* stuff to catch the keyboard */
- (void)keyDown:(NSEvent *)event
{
	NSLog(@"keyDown");
}

- (void)keyUp:(NSEvent *)event
{
	NSLog(@"keyUp");
}

- (void)flagsChanged:(NSEvent *)theEvent
{
	NSLog(@"flagsChanged");
}

- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (void)setupProjection
{
	[[self openGLContext] makeCurrentContext];
	NSRect newFrame = [self frame];

	// set viewport to entire frame
	glViewport(0, 0, newFrame.size.width, newFrame.size.height);

	// setup orthogonal projection with fixed vertical field of view, horizontal adjusted as necessary
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	GLdouble contentRatio = newFrame.size.width / newFrame.size.height;
	glOrtho(-contentRatio, contentRatio, -1, 1, 0.5, 1000);
	glMatrixMode(GL_MODELVIEW);

	glEnable(GL_MULTISAMPLE);
}

- (void)setFrame:(NSRect)frameRect
{
	[super setFrame:frameRect];
	[self setupProjection];
	[self update];

	[[NSNotificationCenter defaultCenter] postNotificationName:@"EBGFSetFirstFrame" object:self];
}

- (void)update:(NSTimer *)aTimer
{
	NSTimeInterval newTime;
	int updates = 0;

	[self setNeedsDisplay:YES];

	newTime = [NSDate timeIntervalSinceReferenceDate];

	if(firstFrame)
	{
		firstFrame = NO;
		updates = 1;
		oldTime = newTime;
	}
	else
	{
		updates = ((newTime - oldTime) / [aTimer timeInterval]) + 0.5;
		oldTime += updates*[aTimer timeInterval];
	}

	while(updates--)
		[delegate updateLogic];
}

- (void)drawRect:(NSRect)dirtyRect
{
	[[self openGLContext] makeCurrentContext];
	[delegate drawFrame];
	glFinish();

	[super drawRect:dirtyRect];
}

- (void)setFirstFrame:(NSNotification *)notification
{
	firstFrame = YES;
}

- (id)initWithFrame:(NSRect)frameRect
{
	// create the pixel format for the GL view
	NSOpenGLPixelFormatAttribute attributes[] =
	{
		NSOpenGLPFASampleBuffers, 1, NSOpenGLPFASamples, 4, NSOpenGLPFASupersample, 0
	};
	NSOpenGLPixelFormat *format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attributes] autorelease];

	if(self = [super initWithFrame:frameRect pixelFormat:format])
	{
		timer = [NSTimer scheduledTimerWithTimeInterval:1.0f/100.0f target:self selector:@selector(update:) userInfo:nil repeats:YES];
		firstFrame = TRUE;
		[self setupProjection];

		GLint swapInt = 1;
		[[self openGLContext] setValues:&swapInt forParameter:NSOpenGLCPSwapInterval];
		
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(setFirstFrame:) name:@"EBGFSetFirstFrame" object:nil];
	}

	return self;
}

- (void)removeFromSuperview
{
	[timer invalidate];
	[self setDelegate:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super removeFromSuperview];
}

@end
